<?php
/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://webappick.com
 * @since      1.0.0
 *
 * @package    Woo_Invoice
 * @subpackage Woo_Invoice/includes
 */

/**
 * The core plugin class that generate PDF Invoice.
 *
 * This is used to generate PDF Invoice
 *
 * @since      1.0.0
 * @package    Woo_Invoice
 * @subpackage Woo_Invoice/includes
 * @author     Md Ohidul Islam <wahid@webappick.com>
 */
class Woo_Invoice_Hooks {



	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   public
	 */
	public function __construct() {

		if ( ! empty( get_option( 'wpifw_invoicing' ) ) ) {

			// ############################################################################################
			// ####################### EMAIL HOOKS START HERE #############################################
			// ############################################################################################
			// Filter to add Invoice attachment with order email.
			if ( ! empty( get_option( 'wpifw_order_email' ) ) ) {
				add_filter( 'woocommerce_email_attachments', array( $this, 'attach_invoice_to_order_email' ), 90, 4 );
				// Filter to add Invoice download link with order email.
				// add_filter( 'woocommerce_email_after_order_table', array( $this, 'add_invoice_download_link' ), 91, 4 );// !
			}
			// ####################### EMAIL HOOKS END HERE #############################################

			// ############################################################################################
			// ### START ### Add Custom MetaBox for PDF Download Button at Admin Order Detail Page ########
			// ############################################################################################
			// Add Custom MetaBox for PDF Download Button.
			add_action( 'add_meta_boxes', array( $this, 'add_custom_meta_box' ) );
			// ##################################### END ##################################################


			// ####################################################################################
			// ##### START ##### WooCommerce Admin Order List Bulk Action Drop-down Hooks. ########
			// ####################################################################################
			// Register bulk Invoice Making action.
			add_filter( 'bulk_actions-edit-shop_order', array( $this, 'register_bulk_invoice_actions' ), 11 );
			// Register bulk Invoice Packing Slip Making action.
			add_filter( 'bulk_actions-edit-shop_order', array( $this, 'register_bulk_packing_slip_actions' ), 11 );
			// Handle bulk Invoice Making action.
			add_filter( 'handle_bulk_actions-edit-post', array( $this, 'woo_invoice_bulk_action_handler' ), 10, 3 );
			// ###################################### END ##########################################

			// ####################################################################################
			// ####### Start ###### WooCommerce Admin Order List Action Buttons Hooks. ############
			// ####################################################################################
			// Add a invoice button on admin orders page.
			add_action( 'woocommerce_admin_order_actions_end', array( $this, 'add_invoice_order_action_button' ) );
			// Add a invoice packing slip button on admin orders page.
			add_action( 'woocommerce_admin_order_actions_end', array( $this, 'add_packing_slip_order_action_button' ) );
			// ###################################### END ##########################################


			// ####################################################################################
			// ######################### MY ACCOUNT HOOKS START HERE ##############################
			// ####################################################################################
			// Add Invoice Download in Order Detail Page.
			if ( ! empty( get_option( 'wpifw_download' ) ) ) {
				add_action( 'woocommerce_order_details_after_order_table', array( $this, 'woo_invoice_download_view_order_page' ) );
			}
			// Filter to add Download Invoice button.
			if ( ! empty( get_option( 'wpifw_download' ) ) ) {
				add_filter( 'woocommerce_my_account_my_orders_actions', array( $this, 'add_my_account_order_action_download_invoice' ), 10, 2 );
			}
			// ################ MY ACCOUNT HOOKS END HERE ###########################################


			// ####################################################################################
			// ################## PDF GENERATING HOOKS START HERE #################################
			// ####################################################################################
			// Generate Invoice.
			add_action( 'wp_ajax_wpifw_generate_invoice', array( $this, 'generate_invoice' ) );
			// Generate Packing Slip.
			add_action( 'wp_ajax_wpifw_generate_invoice_packing_slip', array( $this, 'generate_packing_slip' ) );
			// ################## PDF GENERATING HOOKS END HERE ###################################

			// ####################################################################################
			// ################## PLUGIN SETTINGS HOOKS START HERE ################################
			// ####################################################################################
			// Add invoice number to order.
			// add_action( 'woocommerce_new_order', array( $this, 'add_invoice_number_to_order' ) ); !
			add_action( 'woocommerce_checkout_create_order', array( $this, 'add_invoice_number_to_order' ) );
			// Add LOGO Selection script to admin footer.
			add_action( 'admin_footer', array( $this, 'logo_selector_print_scripts' ) );
			// Template selection hook to save template id/number.
			add_action( 'wp_ajax_wpifw_save_pdf_template', array( $this, 'save_pdf_template' ) );
			// ################## PLUGIN SETTINGS HOOKS END HERE ###################################
		}
	}


	/**
	 * Add Download Invoice button into My Account Order Actions for Customer
	 *
	 * @param array    $actions Order Actions.
	 * @param WC_Order $order Order Object.
	 *
	 * @return mixed
	 */
	public function add_my_account_order_action_download_invoice( $actions, $order ) {

		if ( $order->is_paid() ) {
			$order_id                            = $order->get_id();
			$actions['wpifw-my-account-invoice'] = array(
				// Adjust URL as needed.
				'url'   => wp_nonce_url( admin_url( 'admin-ajax.php?action=wpifw_generate_invoice&order_id=' . $order_id ), 'woo_invoice_ajax_nonce' ),
				'name'  => __( 'Download Invoice', 'woo-invoice' ),
				'class' => 'wpifw_invoice_action_button',
			);
		}

		return $actions;
	}


	/**
	 * Register MetaBox to add PDF Download Button to Admin Order Detail page
	 */
	public function add_custom_meta_box() {
		add_meta_box( 'wpifw-meta-box', 'Woo Invoice', array( $this, 'pdf_meta_box_markup' ), 'shop_order', 'side', 'high', null );
	}


	/**
	 * Add Invoice and Packing Slip Download button to Admin Order Details page Metabox
	 */
	public function pdf_meta_box_markup() {
		global $post;
		$invoice      = wp_nonce_url( admin_url( 'admin-ajax.php?action=wpifw_generate_invoice&order_id=' . $post->ID ), 'woo_invoice_ajax_nonce' );
		$packing_slip = wp_nonce_url( admin_url( 'admin-ajax.php?action=wpifw_generate_invoice_packing_slip&order_id=' . $post->ID ), 'woo_invoice_ajax_nonce' );
		?>
		<div class="wpifw_invoice_info">
			<a href="<?php echo esc_url( $invoice ); ?>" target="_blank">
				<button type="button"
						class="wpifw_button_invoice button wpifw_info_invoice"><?php esc_html_e( 'Invoice', 'woo-invoice' ); ?></button>
			</a>
			<a href="<?php echo esc_url( $packing_slip ); ?>" target="_blank">
				<button type="button"
						class="wpifw_button_invoice_packing_slip button wpifw_info_packing_slip"><?php esc_html_e( 'Packing Slip', 'woo-invoice' ); ?></button>
			</a>
		</div>

		<?php
	}


	/**
	 * Add Invoice & Packing Slip bulk action to order list actions
	 *
	 * @param string $redirect_to URL.
	 * @param string $doaction Action.
	 * @param array  $post_ids Order Ids.
	 * @return string
	 */
	public function woo_invoice_bulk_action_handler( $redirect_to, $doaction, $post_ids ) {
		if ( 'wpifw_bulk_invoice' !== $doaction ) {
			return $redirect_to;
		}
		if ( 'wpifw_bulk_invoice_packing_slip' !== $doaction ) {
			return $redirect_to;
		}
		// foreach ( $post_ids as $post_id ) {
		// Perform action for each post.
		// }//.
		$redirect_to = add_query_arg( 'bulk_emailed_posts', count( $post_ids ), $redirect_to );
		return $redirect_to;
	}

	/**
	 * Register bulk invoice making action
	 *
	 * @param array $bulk_actions Order List Bulk Actions.
	 * @return mixed
	 */
	public function register_bulk_invoice_actions( $bulk_actions ) {
		$bulk_actions['wpifw_bulk_invoice'] = __( 'Make PDF Invoice', 'woo-invoice' );
		return $bulk_actions;
	}

	/**
	 * Register bulk invoice packing slip making action
	 *
	 * @param array $bulk_actions Order List Bulk Actions.
	 * @return mixed
	 */
	public function register_bulk_packing_slip_actions( $bulk_actions ) {
		$bulk_actions['wpifw_bulk_invoice_packing_slip'] = __( 'Make Packing Slip', 'woo-invoice' );
		return $bulk_actions;
	}

	/**
	 * Add a invoice button on admin orders page.
	 *
	 * @param WC_Order $order Order Object.
	 * @return mixed
	 */
	public function add_invoice_order_action_button( $order ) {
		// Get Order ID (compatibility all WC versions).
		$order_id = method_exists( $order, 'get_id' ) ? $order->get_id() : '';

		$url         = wp_nonce_url( admin_url( 'admin-ajax.php?action=wpifw_generate_invoice&order_id=' . $order_id ), 'woo_invoice_ajax_nonce' );
		$text        = __( 'PDF Invoice', 'woo-invoice' );
		$class       = 'tips parcial wpifw_invoice wpifw_invoice_action_button';
		$src         = WOO_INVOICE_FREE_PLUGIN_URL . 'admin/images/invoice.svg';
		$image_class = 'width: 19px;margin-top: 4px;margin-left: 2px;';
		echo '<a href="' . esc_url( $url ) . '" target="_blank" class="button ' . esc_html( $class ) . '" data-tip="' . esc_html( $text ) . '" title="' . esc_html( $text ) . '"> <img src="' . esc_url( $src ) . '" style="' . esc_html( $image_class ) . '"/></a>';
	}

	/**
	 * Add a packing slip action button on admin order list page.
	 *
	 * @param WC_Order $order Order Object.
	 */
	public function add_packing_slip_order_action_button( $order ) {

		// Get Order ID (compatibility all WC versions).
		$order_id = method_exists( $order, 'get_id' ) ? $order->get_id() : '';

		$url         = wp_nonce_url( admin_url( 'admin-ajax.php?action=wpifw_generate_invoice_packing_slip&order_id=' . $order_id ), 'woo_invoice_ajax_nonce' );
		$text        = __( 'Packing Slip', 'woo-invoice' );
		$class       = 'tips parcial wpifw_invoice_packing_slip wpifw_button_invoice_packing_slip';
		$src         = WOO_INVOICE_FREE_PLUGIN_URL . 'admin/images/shipping_list.svg';
		$image_class = 'width: 19px;margin-top: 4px;margin-left: 3px;';
		echo '<a href="' . esc_url( $url ) . '" target="_blank"  class="button ' . esc_html( $class ) . '" data-tip="' . esc_html( $text ) . '" title="' . esc_html( $text ) . '"><img src="' . esc_url( $src ) . '" style="' . esc_html( $image_class ) . '"/></a>';
	}

	/**
	 * Attach Invoice with Order Email
	 *
	 * @param array    $attachments Order Attachments.
	 * @param string   $status order status.
	 * @param WC_Order $order Order Object.
	 * @return array
	 * @throws \Mpdf\MpdfException //phpcs:ignore.
	 */
	public function attach_invoice_to_order_email( $attachments, $status, $order ) {

		if ( ! $order instanceof WC_Order ) {
			return $attachments;
		}

		if ( $order->is_paid() ) {
			$order_id = $order->get_id();

			$allowed_statuses = array(
				'new_order',
				'customer_invoice',
				'customer_processing_order',
				'customer_completed_order',
				'customer_completed_renewal_order',
				'customer_processing_renewal_order',
			);

			woo_invoice_engine()->save_invoice( $order_id );
			$invoice_no = woo_invoice_get_invoice_number( $order_id );
			$file_name  = 'Invoice-' . $invoice_no;
			if ( isset( $status ) && in_array( $status, $allowed_statuses, true ) ) {
				$pdf_path      = WOO_INVOICE_UPLOAD_DIR . $file_name . '.pdf';
				$attachments[] = $pdf_path;
			}
		}

		return $attachments;
	}


	/**
	 * Add Invoice Download link
	 *
	 * @param WC_Order $order Order Object.
	 * @return mixed
	 */
	public function add_invoice_download_link( $order ) {
		$allowed_statuses = array(
			'new_order',
			'customer_invoice',
			'processing',
			'completed',
		);
		$status           = $order->get_status();
		$order_id         = $order->get_id();

		if ( ( isset( $status ) && in_array( $status, $allowed_statuses, true ) ) ) {
			$invoice_download_link = wp_nonce_url( admin_url( 'admin-ajax.php?action=wpifw_generate_invoice&order_id=' . $order_id ), 'woo_invoice_ajax_nonce' );
			$download_invoice_text = get_option( 'wpifw_DOWNLOAD_INVOICE_TEXT' );
			$download_invoice_text = ( $download_invoice_text ) ? $download_invoice_text : 'Download Invoice';
			echo ' <a href="' . esc_url( $invoice_download_link ) . '" target="_blank"  class="button" data-tip="" title="">' . esc_attr( $download_invoice_text ) . '</a> <br><br/>';
		}
		echo '';
	}


	/**
	 * Add Invoice number to order
	 *
	 * @param integer $order_id Order Id.
	 */
	public function add_invoice_number_to_order( $order_id ) {
		$get_prefix = get_option( 'wpifw_invoice_no_prefix' );
		$prefix     = ! empty( $get_prefix ) ? $get_prefix : '';

		$get_suffix = get_option( 'wpifw_invoice_no_suffix' );
		$suffix     = ! empty( $get_suffix ) ? $get_suffix : '';

		$get_next = get_option( 'wpifw_invoice_no' );
		$next_no  = ! empty( $get_next ) ? $get_next : 1;

		$invoice_no = $prefix . $next_no . $suffix;

		update_option( 'wpifw_invoice_no', $next_no + 1 );
		add_post_meta( $order_id, 'wpifw_invoice_no', $invoice_no );
	}


	/**
	 *  Add Logo uploader script to footer
	 */
	public function logo_selector_print_scripts() {

		$my_saved_attachment_post_id = get_option( 'wpifw_logo_selector_attachment_id', 0 );

		?>
		<script type='text/javascript'>

			jQuery(document).ready(function ($) {

				jQuery(document).on("click", "#wpifw_upload_logo_button", function (e) {
					e.preventDefault();
					var $button = $(this);


					// Create the media frame.
					var file_frame = wp.media.frames.file_frame = wp.media({
						title: 'Select or upload image',
						library: { // remove these to show all
							type: 'image' // specific mime
						},
						button: {
							text: 'Select'
						},
						multiple: false  // Set to true to allow multiple files to be selected
					});

					// When an image is selected, run a callback.
					file_frame.on('select', function () {
						// We set multiple to false so only get one image from the uploader

						var attachment = file_frame.state().get('selection').first().toJSON();

						console.log(attachment);

						$('#wpifw_logo-preview').attr('src', attachment.url).css('width', 'auto');
						$('#wpifw_logo_attachment_id').val(attachment.id);

						$button.siblings('input').val(attachment.id).change();

					});

					// Finally, open the modal
					file_frame.open();
				});
			});


		</script>
		<?php
	}

	/**
	 * Add download invoice button in order view actions.
	 * My Account Page
	 *
	 * @param WC_Order $order Order Object.
	 * @return void
	 */
	public function woo_invoice_download_view_order_page( $order ) {
		$order_id = $order->get_id();
		$url      = wp_nonce_url( admin_url( 'admin-ajax.php?action=wpifw_generate_invoice&order_id=' . $order_id ), 'woo_invoice_ajax_nonce' );
		?>
		<a class="woocommerce-button button wpifw-my-account-invoice" href="<?php echo esc_url( $url ); ?>" target="_blank"><?php esc_html_e( 'Download Invoice', 'woo-invoice' ); ?></a>
		<?php
	}

	/**
	 * Generate Invoice
	 */
	public function generate_invoice() {
		if ( ! is_user_logged_in() ) {
			auth_redirect();
			exit;
		}
		$retrieved_nonce = isset( $_REQUEST['_wpnonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['_wpnonce'] ) ) : '';
		if ( ! wp_verify_nonce( $retrieved_nonce, 'woo_invoice_ajax_nonce' ) ) {
			die( 'Failed security check' );
		}

		$order_ids = array();
		if ( isset( $_REQUEST['order_id'] ) ) {
			$order_id = ! empty( $_REQUEST['order_id'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['order_id'] ) ) : '';

			$order = wc_get_order( $order_id );
			$user  = $order->get_user_id();
			if ( wc_user_has_role( get_current_user_id(), 'administrator' ) || get_current_user_id() === $user ) {
				$order_ids = $order_id;
			} else {
				die( 'You are not allowed to download this invoice.' );
			}
		} elseif ( isset( $_REQUEST['order_ids'] ) ) {
			if ( wc_user_has_role( get_current_user_id(), 'administrator' ) || wc_user_has_role( get_current_user_id(), 'shop_manager' ) ) {
				$order_ids = sanitize_text_field( wp_unslash( $_REQUEST['order_ids'] ) );
			} else {
				die( 'You are not allowed to download invoices.' );
			}
		}

		woo_invoice_engine()->generate_invoice( $order_ids );
	}

	/**
	 * Generate Packing Slip
	 */
	public function generate_packing_slip() {
		if ( ! is_user_logged_in() ) {
			auth_redirect();
			exit;
		}
		$retrieved_nonce = isset( $_REQUEST['_wpnonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['_wpnonce'] ) ) : '';
		if ( ! wp_verify_nonce( $retrieved_nonce, 'woo_invoice_ajax_nonce' ) ) {
			die( 'Failed security check' );
		}

		$order_ids = array();
		if ( isset( $_REQUEST['order_id'] ) ) {
			$order_id = ! empty( $_REQUEST['order_id'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['order_id'] ) ) : '';

			$order = wc_get_order( $order_id );
			$user  = $order->get_user_id();
			if ( wc_user_has_role( get_current_user_id(), 'administrator' ) || get_current_user_id() === $user ) {
				$order_ids = $order_id;
			} else {
				die( 'You are not allowed to download this packing slip.' );
			}
		} elseif ( isset( $_REQUEST['order_ids'] ) ) {
			if ( wc_user_has_role( get_current_user_id(), 'administrator' ) || wc_user_has_role( get_current_user_id(), 'shop_manager' ) ) {
				$order_ids = sanitize_text_field( wp_unslash( $_REQUEST['order_ids'] ) );
			} else {
				die( 'You are not allowed to download packing slip.' );
			}
		}
		woo_invoice_engine()->generate_packing_slip( $order_ids );
	}

	/**
	 * Save invoice template number from plugin settings
	 */
	public function save_pdf_template() {
		check_ajax_referer( 'woo_invoice_ajax_nonce' );
		$template = isset( $_POST['template'] ) ? sanitize_text_field( wp_unslash( $_POST['template'] ) ) : '';
		if ( !empty( $template ) ) {
			update_option( 'wpifw_templateid', $template );
			wp_send_json_success( $template );
		}
		wp_send_json_error($_POST);
	}
}

new Woo_Invoice_Hooks();
