<?php
/**
 * WooCommerce Invoice Free Plugin Uses Tracker
 * Uses Webappick Insights for tracking
 * @since 1.2.2
 * @version 1.0.0
 */
if( ! defined( 'ABSPATH' ) ) die();

if (!class_exists('WooInvoiceTracker')) {
    /**
     * Class WooInvoiceTracker
     */
    final class WooInvoiceTracker {

        /**
         * Singleton instance
         * @var WooInvoiceTracker
         */
        protected static $instance;

        /**
         * @var WebAppick\AppServices\Insights
         */
        protected $insights = null;

        /**
         * Promotions Class Instance
         * @var \WebAppick\AppServices\Promotions
         */
        public $promotion;

        /**
         * @var WebAppick\AppServices\Updater
         */
        protected $updater = null;

        /**
         * @var WebAppick\AppServices\License
         */
        protected $license = null;

        /**
         * Initialize
         * @return WooInvoiceTracker
         */
        public static function getInstance()
        {
            if (is_null(self::$instance)) self::$instance = new self();
            return self::$instance;
        }

        /**
         * Class constructor
         *
         * @return void
         * @since 1.0.0
         *
         */
        private function __construct()
        {
            add_action('init', [$this, '__init']);
        }

        /**
         * Cloning is forbidden.
         * @since 1.0.2
         */
        public function __clone()
        {
            _doing_it_wrong(__FUNCTION__, __('Cloning is forbidden.', 'woo-invoice'), '1.0.2');
        }

        public function __init()
        {
            if (!class_exists('WebAppick\AppServices\Client')) {
                /** @noinspection PhpIncludeInspection */
                require_once WAPDFINVOICE_LIBS_PATH . 'WebAppick/AppServices/Client.php';
            }

            $client = new WebAppick\AppServices\Client( '28261c3c-793e-4c74-bf7f-2837b8c6ddb4', 'Woo Invoice Free', WAPDFINVOICE_FREE_FILE );

            $this->insights = $client->insights();

            $this->insights->hide_notice();

            // Promo offers
            $this->promotion = $client->promotions();
            $this->promotion->set_source('https://api.bitbucket.org/2.0/snippets/woofeed/RLbyop/files/woo-feed-notice.json')->init();

            add_filter($client->getSlug() . '_what_tracked', [$this, 'data_we_collect'], 10, 1);

            $this->insights->init_plugin();
        }

        /**
         * Set Data Collection description for the tracker
         * @param $data
         *
         * @return array
         */
        public function data_we_collect($data)
        {
            $data = array_merge($data, [
                esc_html__('Number of products in your site.', 'woo-invoice'),
                esc_html__('Site name, language and url.', 'woo-invoice'),
                esc_html__('Number of active and inactive plugins.', 'woo-invoice'),
                esc_html__('Your name and email address.', 'woo-invoice')
            ]);
            return $data;
        }

        public function get_data_collection_description()
        {
            return $this->insights->get_data_collection_description();
        }

        /**
         * Update Tracker OptIn
         *
         * @param bool $override optional. ignore last send datetime settings if true.
         * @return void
         * @see Insights::send_tracking_data()
         */
        public function trackerOptIn($override = false)
        {
            $this->insights->optIn($override);
        }

        /**
         * Update Tracker OptOut
         * @return void
         */
        public function trackerOptOut()
        {
            $this->insights->optOut();
        }

        /**
         * Check if tracking is enable
         * @return bool
         */
        public function is_tracking_allowed()
        {
            return $this->insights->is_tracking_allowed();
        }
    }
}